from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="ModelRunResponse")


@attr.s(auto_attribs=True)
class ModelRunResponse:
    """
    Attributes:
        slug (str): Short identifying name for the model Example: var.
        version (str): Version of the model Example: 1.0.
        chain_id (float): Chain id
        block_number (float): Block number
        block_timestamp (float): Block timestamp
        output (Dict[str, Any]): Data generated by the model. An object, the contents of which are specific to the
            model. Example: { ... }.
        dependencies (Dict[str, Any]): Dictionary of model dependencies, model name to version or version list. Example:
            {"x": "1.0", "y": ["2.0", "2.1"]}.
        cached (bool): Whether it is a cached result Example: True.
        runtime (float): Running time of the model in milliseconds. Example: 5100.
    """

    slug: str
    version: str
    chain_id: float
    block_number: float
    block_timestamp: float
    output: Dict[str, Any]
    dependencies: Dict[str, Any]
    cached: bool
    runtime: float

    def to_dict(self) -> Dict[str, Any]:
        slug = self.slug
        version = self.version
        chain_id = self.chain_id
        block_number = self.block_number
        block_timestamp = self.block_timestamp
        output = self.output
        dependencies = self.dependencies
        cached = self.cached
        runtime = self.runtime

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "slug": slug,
                "version": version,
                "chainId": chain_id,
                "blockNumber": block_number,
                "blockTimestamp": block_timestamp,
                "output": output,
                "dependencies": dependencies,
                "cached": cached,
                "runtime": runtime,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        slug = d.pop("slug")

        version = d.pop("version")

        chain_id = d.pop("chainId")

        block_number = d.pop("blockNumber")

        block_timestamp = d.pop("blockTimestamp")

        output = d.pop("output")

        dependencies = d.pop("dependencies")

        cached = d.pop("cached")

        runtime = d.pop("runtime")

        model_run_response = cls(
            slug=slug,
            version=version,
            chain_id=chain_id,
            block_number=block_number,
            block_timestamp=block_timestamp,
            output=output,
            dependencies=dependencies,
            cached=cached,
            runtime=runtime,
        )

        return model_run_response
